/**********************************************************
 * WebMailFolder                                          *
 * All rights reserved                                    *
 * 1996/08/16   Norbert Kuemin   norbert.kuemin@lugs.ch   *
 **********************************************************/

#define MAIL_FILE
#include "wmf.h"

int IsEmailChar(char chr) {
    if ((chr >= 'A') && (chr <= 'Z')) return TRUE;
    if ((chr >= 'a') && (chr <= 'z')) return TRUE;
    if ((chr >= '0') && (chr <= '9')) return TRUE;
    if (chr == '-') return TRUE;
    if (chr == '_') return TRUE;
    return FALSE;
}


char *GetHeadData(char *line) {
    char *c;
    char *data;
    int i;

    c = line;
    while (c[0] != ':') c++;
    c++;
    while (c[0] == ' ') c++;
    i = 0;
    while ((c[i] != '\n') && (c[i] != '\0'))
        i++;
    data = newchar(i+1);
    strncpy(data,c,i);
    data[i] = '\0';
    return data;
}


char *GetName(char *line) {
    char *name;
    char *c;
    int i;
        

    if (config.DebugLevel & 1)
        printf(".GetName\n");
    switch (line[strlen(line)-1]) {
	case '>': /* name <email> */
	    c = line;
	    /* Calculate length */
	    i = 0;
	    while ((c[i] != '<') && (c[i] != '\0'))
	        i++;
	    i--;
	    /* Remove spaces at the end */
	    while ((c[i] == ' '))
	        i--;
	    i++;
	    name = newchar(i+1);
	    strncpy(name,c,i);
            name[i] = '\0';
	    break;
	case ')': /* email (name) */
	    c = line;
	    /* Start at ( */
	    while (c[0] != '(') c++;
	    c++;
	    /* Remove all leading spaces */
	    while (c[0] == ' ') c++;
	    /* Calculate length */
	    i = 0;
	    while ((c[i] != ')') && (c[i] != '\0'))
	        i++;
	    i--;
	    /* Remove spaces at the end */
	    while ((c[i] == ' '))
	        i--;
	    i++;
	    name = newchar(i+1);
	    strncpy(name,c,i);
            name[i] = '\0';
	    break;
	default: /* Email only */
	    name = "\0";
	    break;
    }
    return name;
}


char *GetEmail(char *line) {
    char *email;
    char *c;
    int i;

    if (config.DebugLevel & 1)
        printf(".GetEmail\n");        
    switch (line[strlen(line)-1]) {
	case '>': /* name <email> */
	    c = line;
	    while (c[0] != '<') c++;
	    c++;
	    /* Calculate length */
	    i = 0;
	    while ((c[i] != '>') && (c[i] != '\0'))
	        i++;
	    email = newchar(i+1);
	    strncpy(email,c,i);
            email[i] = '\0';
	    break;
	case ')': /* email (name) */
	    c = line;
	    /* Calculate length */
	    i = 0;
	    while ((c[i] != '(') && (c[i] != '\0'))
	        i++;
	    i--;
	    /* Remove spaces at the end */
	    while ((c[i] == ' '))
	        i--;
	    i++;
	    email = newchar(i+1);
	    strncpy(email,c,i);
            email[i] = '\0';
	    break;
	default: /* Email only */
            email = dupchar(line);
	    break;
    }
    return email;
}


char *GetMsgId(char *line) {
    char *id;
    char *c;
    int i;
        

    if (config.DebugLevel & 1)
        printf(".GetMsgId(%s)\n",line);
    c = line;
    /* Start at < */
    while (c[0] != '<') {
        if (c[0] == 0) break;
        c++;
    }
    if (c[0] != 0) c++;
    /* Calculate length */
    i = 0;
    while ((c[i] != '>') && (c[i] != '\0'))
        i++;
    id = newchar(i+1);
    strncpy(id,c,i);
    id[i] = '\0';
    return id;
}


char *AddData(char *line, char *data) {
    char *newdata;
    char *c;
    int i;
        

    if (config.DebugLevel & 1)
        printf(".AddData\n");

    c = line;
    /* remove spaces and tabs at start */
    while ((c[0] == ' ') || (c[0] == '\t')) c++;
    /* Calculate length */
    i = 0;
    while ((c[i] != '\n') && (c[i] != '\0'))
        i++;
    if ((line[0] == '\t') || (strncmp(line,"        ",8) == 0)) {
	newdata = newchar(i+strlen(data)+2);
    } else {
	newdata = newchar(i+strlen(data)+1);
    }
    strcpy(newdata,data);
    if ((line[0] == '\t') || (strncmp(line,"        ",8) == 0))
        strcat(newdata," ");
    strncat(newdata,c,i);
    strcat(newdata,"\0");
    return newdata;
}


struct MAILBODY *addbody(char *line, struct MAILBODY *body) {

    if (body->Line == NULL) {
	body->Line = dupchar(line);
	body->Next = NULL;
	return body;
    } else {
	body->Next = (struct MAILBODY *) malloc(sizeof(struct MAILBODY));
	body->Next->Line = dupchar(line);
	body->Next->Next = NULL;
	return body->Next;
    }
}


void GetMail(FILE *fd, struct MAILHEAD *mh, struct MAILBODY *mb) {
    char line[BUFSIZ];
    char *tmpFrom, *tmpTo, *tmpId, *tmpRef, *tmpstr;
    struct MAILBODY *body;
    int status; /* -1=Error 0=End 1=Header 2=Body */
    int lastlength;
    int lastheadcmd; /*-1=Unknown 0=None 1=From 2=To 3=Subject 4=Date
    			5=Message-Id 6=Reference 7=Xface 8=HomePage*/
    
    if (config.DebugLevel & 1)
        printf(".GetMail\n");
    body = mb;
    body->Line = NULL;
    body->Next = NULL;
    lastheadcmd = 0;
    status = 1;
    lastlength = 0;
    tmpFrom = NULL;
    tmpTo = NULL;
    tmpId = NULL;
    tmpRef = NULL;
    mh->SendName = "\0";
    mh->SendAddress = "\0";
    mh->ReceiveName = "\0";
    mh->ReceiveAddress = "\0";
    mh->MsgId = "\0";
    mh->Subject = "\0";
    mh->Reference = "\0";
    mh->Xface = "\0";
    mh->HomePage = "\0";
    while (status > 0) {
        fgets(line,BUFSIZ,fd);
	switch (status) {
	    case 1:
	        /* Header */
		if ((strlen(line) == 0) || (line[0] == '\n')) {
		    /* An empty line indicates the end of the header */
		    status = 2;
		    if (tmpFrom != NULL) {
			mh->SendAddress = GetEmail(tmpFrom);
			mh->SendName = GetName(tmpFrom);
			freechar(tmpFrom);
		    }
		    if (tmpTo != NULL) {
			mh->ReceiveAddress = GetEmail(tmpTo);
			mh->ReceiveName = GetName(tmpTo);
			freechar(tmpTo);
		    }
		    if (tmpId != NULL) {
			mh->MsgId = GetMsgId(tmpId);
			freechar(tmpId);
		    }
		    if (tmpRef != NULL) {
			mh->Reference = GetMsgId(tmpRef);
			freechar(tmpRef);
		    }
		}
		if ((line[0] == ' ') || (line[0] == '\t')) {
		    /* A space or tab means add info to previous line */
		    switch (lastheadcmd) {
			case 1:
			    tmpstr = AddData(line,tmpFrom);
			    freechar(tmpFrom);
			    tmpFrom = tmpstr;
			    break;
			case 2:
			    tmpstr = AddData(line,tmpTo);
			    freechar(tmpTo);
			    tmpTo = tmpstr;
			    break;
			case 3:
			    tmpstr = AddData(line,mh->Subject);
			    freechar(mh->Subject);
			    mh->Subject = tmpstr;
			    break;
			case 4: break; /* Unknown to handle */
			case 5:
			    tmpstr = AddData(line,tmpId);
			    freechar(tmpId);
			    tmpId = tmpstr;
			    break;
			case 6:
			    tmpstr = AddData(line,tmpRef);
			    freechar(tmpRef);
			    tmpRef = tmpstr;
			    break;
			case 7:
			    tmpstr = AddData(line,mh->Xface);
			    freechar(mh->Xface);
			    mh->Xface = tmpstr;
			    break;
			case 8:
			    tmpstr = AddData(line,mh->HomePage);
			    freechar(mh->HomePage);
			    mh->HomePage = tmpstr;
			    break;
		    }
		} else {
		    lastheadcmd = -1;
		}
		if (strncmp(line,"From:",5) == 0) {
		    tmpFrom = GetHeadData(line);
		    lastheadcmd = 1;
		}
		if (strncmp(line,"To:",3) == 0) {
		    tmpTo = GetHeadData(line);
		    lastheadcmd = 2;
		}
		if (strncmp(line,"Subject:",8) == 0) {
		    mh->Subject = GetHeadData(line);
		    lastheadcmd = 3;
		}
		if (strncmp(line,"Date:",5) == 0) {
		    mh->SentDate = GetHeadData(line);
		    lastheadcmd = 4;
		}
		if ((strncmp(line,"Message-Id:",11) == 0) ||
		    (strncmp(line,"Message-ID:",11) == 0)) {
		    tmpId = GetHeadData(line);
		    lastheadcmd = 5;
		}
		if ((strncmp(line,"In-Reply-To:",12) == 0) ||
		    (strncmp(line,"References:",11) == 0)) {
		    tmpRef = GetHeadData(line);
		    lastheadcmd = 6;
		}
		if ((strncmp(line,"X-face:",7) == 0) ||
		    (strncmp(line,"X-Face:",7) == 0)) {
		    mh->Xface = GetHeadData(line);
		    lastheadcmd = 7;
		}
		if (strncmp(line,"X-URL:",6) == 0) {
		    mh->HomePage = GetHeadData(line);
		    lastheadcmd = 8;
		}
		break;
	    case 2:
	        /* Body */
		if (lastlength == 0) {
		    if (strncmp(line,"From ",5) == 0) {
			/* Next mail detected */
			status = 0;
		    } else {
			/* It was only an empty line */
			body = addbody("\n",body);
		    }
		}
		lastlength = strlen(line);
		if (line[0] == '\n') lastlength = 0;
		if ((lastlength != 0) && (status>0)) {
		    body = addbody(line,body);
		}
		break;
	}
	if(feof(fd) != 0) status=0;
    }
}


void WriteMailBody(char *FileName, char *Data) {
    char *datastr, *tmpstr, *tmpmail, tmpchar;
    int tmpint, n, m;
    
    datastr = dupchar(Data);
    /* quoted data */
    /* remove = at the end of a line */
    if (strlen(datastr) > 2) {
	if ((datastr[strlen(datastr)-2] == '=') && 
	  (datastr[strlen(datastr)-3] == ' ')) {
            tmpint = 2;
            while (datastr[strlen(datastr)-tmpint-1] == ' ') tmpint++;
	    tmpstr = newchar(strlen(datastr)-tmpint+2);
	    strncpy(tmpstr,datastr,strlen(datastr)-tmpint);
	    tmpstr[strlen(datastr)-tmpint] = ' ';
	    tmpstr[strlen(datastr)-tmpint+1] = '\0';
	    freechar(datastr);
	    datastr = tmpstr;
	}
    }

    /* quoted mail never will be greater than the original */
    tmpstr = newchar(strlen(datastr)+1);
    tmpstr[0] = '\0';

    for(n=0;n<(strlen(datastr));n++) {
	tmpchar = datastr[n];
        if ((datastr[n] == '=') && (n < (strlen(datastr)-2)))
	    if (((datastr[n+1] >= '0') && (datastr[n+1] <= '9')) || 
	      ((datastr[n+1] >= 'A') && (datastr[n+1] <= 'F')))
		if (((datastr[n+2] >= '0') && (datastr[n+2] <= '9')) || 
		  ((datastr[n+2] >= 'A') && (datastr[n+2] <= 'F'))) {
		    if (datastr[n+1] > '9')
			tmpchar = 16 * (datastr[n+1]-55);
		    else
			tmpchar = 16 * (datastr[n+1]-48);
		    if (datastr[n+2] > '9')
			tmpchar = tmpchar + (datastr[n+2]-55);
		    else
			tmpchar = tmpchar + (datastr[n+2]-48);
		    n = n + 2;
		}
	tmpstr[strlen(tmpstr)+1] = '\0';
	tmpstr[strlen(tmpstr)] = tmpchar;
    }
    freechar(datastr);
    datastr = tmpstr;

    /* url hyperlinks */
    /* url mail never will be greater than 10 times the original length */
    tmpstr = newchar(strlen(datastr)*10);
    tmpstr[0] = '\0';
    for(n=0;n<(strlen(datastr));n++) {
	tmpchar = 0;
	for(m=0 ; protocols[m] != NULL ; m++) {
	    if (strncmp((datastr+n),protocols[m],strlen(protocols[m])) == 0) {
		tmpchar = 0;
	        if (n > 0) tmpchar = datastr[n-1];
		if (tmpchar != '"') tmpchar = ' ';
		tmpint = 0;
		strcat(tmpstr,"<A HREF=\"");
		for (;;) {
		    if (tmpchar == '"') {
			if (datastr[n+tmpint] == '"') break;
		    } else {
			if (datastr[n+tmpint] == '\n') break;
			if (datastr[n+tmpint] == '\0') break;
			if (datastr[n+tmpint] == '\t') break;
			if (datastr[n+tmpint] == ' ') break;
		    }
		    tmpint++;
		}
		strncat(tmpstr,datastr+n,tmpint);
		strcat(tmpstr,"\">");
		strncat(tmpstr,datastr+n,tmpint);
		strcat(tmpstr,"</A>");
		n = n + tmpint;
	    }
	}
	if (n < strlen(datastr)) {
	    tmpstr[strlen(tmpstr)+1] = '\0';
	    tmpstr[strlen(tmpstr)] = datastr[n];
	}
    }
    freechar(datastr);
    datastr = dupchar(tmpstr);
    freechar(tmpstr);

    /* mail hyperlinks */
    /* mail never will be greater than 10 times the original length */

    if ((strstr(datastr,"@") != NULL) && (config.MailCommand != NULL)) {
    	tmpstr = newchar(strlen(datastr)*10);
	tmpstr[0] = '\0';
	tmpmail = datastr;
	while (tmpmail != NULL) {
	    n=0;
	    while ((tmpmail[n] != '@') && (tmpmail[n] != '\0')) n++;
	    if (tmpmail[n] == '\0') {
		/* Append the rest of the line */
		strcat(tmpstr,tmpmail);
		tmpmail = NULL;
	    } else {
		/* Found a @, now check if it is an email */
		if ((n>0) && (tmpmail[n+1] != '\n')) {
		    if (IsEmailChar(tmpmail[n-1]) && IsEmailChar(tmpmail[n+1])) {
			/* Real email found */
			/* Move to the beginning */
			n--;
			while ((n>=0) && (IsEmailChar(tmpmail[n]) ||
			    (tmpmail[n]=='.'))) n--;
			n++;
			if (n > 0) {
			    strncat(tmpstr,tmpmail,n);
			    tmpmail = tmpmail + n;
			}
			/* Move to the ending */
			n = 0;
			while ((n>=0) && (IsEmailChar(tmpmail[n]) ||
			    (tmpmail[n]=='.') || (tmpmail[n]=='@'))) n++;
			strcat(tmpstr,"<A HREF=\"");
			strcat(tmpstr,config.MailCommand);
			strncat(tmpstr,tmpmail,n);
			strcat(tmpstr,"\">");
			strncat(tmpstr,tmpmail,n);
			strcat(tmpstr,"</A>");
			tmpmail = tmpmail + n;
		    } else {
			/* Seams to be anything else */
			n++;
			strncat(tmpstr,tmpmail,n);
			tmpmail = tmpmail + n;
		    }
		} else {
		    /* Found a @ at the begin or the end of the line */
		    n++;
		    strncat(tmpstr,tmpmail,n);
		    tmpmail = tmpmail + n;
		}
	    }
	}
	freechar(datastr);
	datastr = dupchar(tmpstr);
	freechar(tmpstr);
    }

    /* Write line */
    AppendFile(FileName,datastr);
    freechar(datastr);

}

void WriteMail (char *Filename, struct MAILHEAD *mh, struct MAILBODY *mb) {
    char *tmpstr, *tmpmail;
    struct MAILBODY *tmpmb;
    size_t n;
    
    if (config.DebugLevel & 1)
        printf(".WriteMail\n");
    InitHtml(Filename);

    if (config.DebugLevel & 2)
        printf(".Write head\n");
    /* Write headdata as remark */
    WriteHtmlValue(Filename,"Version",HTML_VERSION);
    WriteHtmlValue(Filename,"Subject",mh->Subject);
    WriteHtmlValue(Filename,"FromName",mh->SendName);
    WriteHtmlValue(Filename,"FromEmail",mh->SendAddress);
    WriteHtmlValue(Filename,"ToName",mh->ReceiveName);
    WriteHtmlValue(Filename,"ToEmail",mh->ReceiveAddress);
    WriteHtmlValue(Filename,"Date",mh->SentDate);
    WriteHtmlValue(Filename,"Id",mh->MsgId);
    WriteHtmlValue(Filename,"Reference",mh->Reference);
    WriteHtmlValue(Filename,"X-Face",mh->Xface);
    WriteHtmlValue(Filename,"X-URL",mh->HomePage);

    /* Write html data */
    AppendFile(Filename,"<HEAD>\n");
    if (config.Name == NULL) {
	tmpstr = newchar(strlen(mh->Subject)+17);
	sprintf(tmpstr,"<TITLE>%s</TITLE>\n",mh->Subject);
    } else {
	tmpstr = newchar(strlen(config.Name)+strlen(mh->Subject)+19);
	sprintf(tmpstr,"<TITLE>%s: %s</TITLE>\n",config.Name,mh->Subject);
    }
    AppendFile(Filename,tmpstr);
    freechar(tmpstr);
    AppendFile(Filename,"</HEAD>\n");

    AppendFile(Filename,"<BODY>\n");

    WriteHtmlValue(Filename,"IndexControl1","Start");
    AppendFile(Filename,"Index:\n");
    AppendFile(Filename,"[<A HREF=\"thread.html\">thread</A>]\n");
    AppendFile(Filename,"[<A HREF=\"date.html\">date</A>]\n");
    AppendFile(Filename,"[<A HREF=\"subject.html\">subject</A>]\n");
    AppendFile(Filename,"[<A HREF=\"author.html\">author</A>]\n");
    AppendFile(Filename,"<HR>\n");
    WriteHtmlValue(Filename,"IndexControl1","End");

    WriteHtmlValue(Filename,"Header","Start");
    AppendFile(Filename,"<PRE>\n");

#ifdef FACES
    /* Include face picture */
    if (strlen(mh->Xface) > 0) {
	tmpstr = newchar(strlen(mh->SendAddress)+5);
	sprintf(tmpstr,"%s.xbm",mh->SendAddress);
	AppendFile(Filename,"<IMG ALIGN=RIGHT SRC=\"");
	AppendFile(Filename,tmpstr);
	AppendFile(Filename,"\">");
	freechar(tmpstr);
    }
#endif

    /* From */
    if (config.MailCommand == NULL) {
        if (strlen(mh->SendName) == 0) {
	    tmpstr = newchar(strlen(mh->SendAddress)+10);
	    sprintf(tmpstr,"  From: %s",mh->SendAddress);
        } else {
	    tmpstr = newchar(strlen(mh->SendName)+strlen(mh->SendAddress)+20);
	    sprintf(tmpstr,"  From: %s <I>(%s)</I>",mh->SendName,mh->SendAddress);
        }
        AppendFile(Filename,tmpstr);
        freechar(tmpstr);
    } else {
        if (strlen(mh->SendName) == 0) {
	    tmpstr = newchar(strlen(mh->SendAddress)*2+
	        strlen(config.MailCommand)+25);
	    sprintf(tmpstr,"  From: <A HREF=\"%s%s\">%s</A>",
	        config.MailCommand,mh->SendAddress,mh->SendAddress);
        } else {
	    tmpstr = newchar(strlen(mh->SendName)+strlen(mh->SendAddress)*2+strlen(config.MailCommand)+35);
	    sprintf(tmpstr,"  From: %s <I>(<A HREF=\"%s%s\">%s</A>)</I>",
	        mh->SendName,config.MailCommand,mh->SendAddress,
	        mh->SendAddress);
        }
        AppendFile(Filename,tmpstr);
        freechar(tmpstr);
    }
    if (strlen(mh->HomePage) > 0) {
	tmpstr = newchar(strlen(mh->HomePage)+30);
	sprintf(tmpstr," <A HREF=\"%s\">WWW-Homepage</A>\n",
	    mh->HomePage);
        AppendFile(Filename,tmpstr);
        freechar(tmpstr);
    } else {
        AppendFile(Filename,"\n");
    }

    /* To */
    if (config.MailCommand == NULL) {
        if (strlen(mh->ReceiveName) == 0) {
	    tmpstr = newchar(strlen(mh->ReceiveAddress)+9);
	    sprintf(tmpstr,"  To  : %s\n",mh->ReceiveAddress);
        } else {
	    tmpstr = newchar(strlen(mh->ReceiveName)+strlen(mh->ReceiveAddress)+20);
	    sprintf(tmpstr,"  To  : %s <I>(%s)</I>\n",mh->ReceiveName,mh->ReceiveAddress);
        }
        AppendFile(Filename,tmpstr);
        freechar(tmpstr);
    } else {
	if (strstr(mh->ReceiveAddress,",") != NULL) {
	    /* Multiple address */
	    tmpstr = newchar(strlen(mh->ReceiveAddress)*10+
		strlen(config.MailCommand)+25);
	    sprintf(tmpstr,"  To  : ");
	    tmpmail = mh->ReceiveAddress;
	    while (tmpmail != NULL) {
	        /* remove leading spaces */
	        while (tmpmail[0] == ' ') tmpmail++;
	        n=0;
		while ((tmpmail[n] != '\0') && (tmpmail[n] != ',')) n++;
		strcat(tmpstr,"<A HREF=\"");
		strcat(tmpstr,config.MailCommand);
		strncat(tmpstr,tmpmail,n);
		strcat(tmpstr,"\">");
		strncat(tmpstr,tmpmail,n);
		strcat(tmpstr,"</A>");
		if (tmpmail[n] == ',') {
		    strcat(tmpstr,", ");
		    tmpmail = tmpmail + n + 1;
		} else {
		    strcat(tmpstr,"\n");
		    tmpmail = NULL;
		}
	    }
	} else {
	    /* Single address */
	    if (strlen(mh->ReceiveName) == 0) {
		tmpstr = newchar(strlen(mh->ReceiveAddress)*2+
		    strlen(config.MailCommand)+25);
		sprintf(tmpstr,"  To  : <A HREF=\"%s%s\">%s</A>\n",
		    config.MailCommand,mh->ReceiveAddress,mh->ReceiveAddress);
 	    } else {
		tmpstr = newchar(strlen(mh->ReceiveName)+
		    strlen(mh->ReceiveAddress)*2+strlen(config.MailCommand)+35);
		sprintf(tmpstr,"  To  : %s <I>(<A HREF=\"%s%s\">%s</A>)</I>\n",
		    mh->ReceiveName,config.MailCommand,mh->ReceiveAddress,
		    mh->ReceiveAddress);
	    }
	}
        AppendFile(Filename,tmpstr);
        freechar(tmpstr);
    }
    tmpstr = newchar(strlen(mh->SentDate)+10);
    sprintf(tmpstr,"  Date: %s\n",mh->SentDate);
    AppendFile(Filename,tmpstr);
    freechar(tmpstr);
    AppendFile(Filename,"</PRE>\n");

    /* Subject as title */
    tmpstr = newchar(strlen(mh->Subject)+11);
    sprintf(tmpstr,"<H1>%s</H1>\n",mh->Subject);
    AppendFile(Filename,tmpstr);
    freechar(tmpstr);
    WriteHtmlValue(Filename,"Header","End");

    if (config.DebugLevel & 2)
        printf(".Write body\n");
    WriteHtmlValue(Filename,"Body","Start");
    AppendFile(Filename,"<PRE>\n");
    tmpmb = mb;
    /* Remove leading empty lines */
    while (tmpmb->Line != NULL && tmpmb->Line[0] == '\n') {
	tmpmb = tmpmb->Next;
    }
    while (tmpmb != NULL && tmpmb->Line != NULL) {
        WriteMailBody(Filename,tmpmb->Line);
	tmpmb = tmpmb->Next;
    }
    AppendFile(Filename,"</PRE>\n");
    WriteHtmlValue(Filename,"Body","End");

    WriteHtmlValue(Filename,"IndexControl2","Start");
    AppendFile(Filename,"<HR>\n");
    AppendFile(Filename,"Index:\n");
    AppendFile(Filename,"[<A HREF=\"thread.html\">thread</A>]\n");
    AppendFile(Filename,"[<A HREF=\"date.html\">date</A>]\n");
    AppendFile(Filename,"[<A HREF=\"subject.html\">subject</A>]\n");
    AppendFile(Filename,"[<A HREF=\"author.html\">author</A>]\n");
    WriteHtmlValue(Filename,"IndexControl2","End");
    WriteHtmlValue(Filename,"ThreadControl","Start");
    WriteHtmlValue(Filename,"ThreadControl","End");

    AppendFile(Filename,"</BODY>\n");

    EndHtml(Filename);
}


void CleanMailMem(struct MAILHEAD *mh, struct MAILBODY *mb) {
    struct MAILBODY *tmpmb, *tmpmbnext;

    if (config.DebugLevel & 1)
        printf(".CleanMailMem\n");
    if (mh->SendName[0] != 0) freechar(mh->SendName);
    if (mh->SendAddress[0] != 0) freechar(mh->SendAddress);
    if (mh->ReceiveName[0] != 0) freechar(mh->ReceiveName);
    if (mh->ReceiveAddress[0] != 0) freechar(mh->ReceiveAddress);
    if (mh->SentDate[0] != 0) freechar(mh->SentDate);
    if (mh->MsgId[0] != 0) freechar(mh->MsgId);
    if (mh->Reference[0] != 0) freechar(mh->Reference);
    if (mh->Subject[0] != 0) freechar(mh->Subject);
    if (mh->Xface[0] != 0) freechar(mh->Xface);
    if (mh->HomePage[0] != 0) freechar(mh->HomePage);

    if (mb->Line != NULL) freechar(mb->Line);
    tmpmb = mb->Next;
    while (tmpmb != NULL) {
	if (tmpmb->Line != NULL) freechar(tmpmb->Line);
	tmpmbnext = tmpmb->Next;
	free (tmpmb);
	tmpmb = tmpmbnext;
    }
}

#ifdef FACES
void WriteXFace(struct MAILHEAD *mh) {
    char buffer[4096], line[255];
    char *FileName;
    int byte1, byte2, BitmapCount, BufferCount;

    if (config.DebugLevel & 1)
        printf(".WriteXFace (%s)\n",mh->SendAddress);
    /* Calculate full filename */
    FileName = newchar(strlen(config.Directory)+strlen(mh->SendAddress)+6);
    sprintf(FileName,"%s/%s.xbm",config.Directory,mh->SendAddress);

    /* Getting bitmap */
    strcpy(buffer, mh->Xface);
    switch (uncompface(buffer)) {
	case -2 :
	    fprintf(stderr,"internal compface error (%s)\n",mh->FileName);
	    return;
	    break;
	case -1 :
	    fprintf(stderr,"bad face data (%s)\n",mh->FileName);
	    return;
	    break;
	case 1 :
	    fprintf(stderr,"ignoring trailing garbage (%s)\n",mh->FileName);
	    return;
	    break;
    }

    /* Writing XBM-Header */
    sprintf(line,"#define %s_width %d\n",mh->SendAddress,WIDTH);
    NewFile(FileName,line);
    sprintf(line,"#define %s_height %d\n",mh->SendAddress,HEIGHT);
    AppendFile(FileName,line);
    sprintf(line,"static char %s_bits[] = {\n",mh->SendAddress);
    AppendFile(FileName,line);

    /* Writing data */
    BitmapCount = BufferCount = 0;
    while (buffer[BufferCount]) {
	if (sscanf (buffer + BufferCount, " 0x%2x%2x , ", &byte1, &byte2) != 2)
	    break;
	if (BitmapCount > 11) {
	    AppendFile(FileName,",\n");
	    BitmapCount = 0;
	}
        if (BitmapCount == 0) AppendFile(FileName,"   ");
        if (BitmapCount > 0) AppendFile(FileName,", ");
	while (buffer[BufferCount++] != ',');
	if (byte1 > 255 || byte2 > 255) return;
	sprintf(line,"0x%.2x, 0x%.2x",FaceConvertTable[byte1],FaceConvertTable[byte2]);
	AppendFile(FileName,line);
	BitmapCount = BitmapCount + 2;
    }

    /* Writing XBM-Foot */
    AppendFile(FileName,"};\n");
    freechar(FileName);
}
#endif


void Mail2Html(char *mbox) {
    FILE *fd;
    int FileNumber;
    struct MAILHEAD mh;
    struct MAILBODY mb;
    char *FileName;
        
    if (config.DebugLevel & 1)
        printf(".Mail2Html(%s)\n",mbox);
    if (config.WriteMode == HTML_INDEX) {
        if (config.DebugLevel & 2)
            printf("..IndexOnly -> abort\n");
        return;
    }
    if (strcmp(mbox,"-") != 0) {
        if (!FileExist(mbox)) {
            fprintf(stderr,"* file %s doesn't exist. *\n",mbox);
            return;
        }
        fd = fopen(mbox,"r");
	if (fd == NULL) {
            fprintf(stderr,"* unable to open file %s. *\n",mbox);
            return;
	}
    } else {
        fd = stdin;
    }
    if (config.WriteMode == HTML_APPEND) {
        FileNumber=0;
    } else {
	/* delete all files in the html-directory */
	CleanDirectory(config.Directory);
        FileNumber=-1;
    }
    while (feof(fd) == 0) {
	GetMail(fd,&mh,&mb);
        if (config.WriteMode == HTML_APPEND) {
	    FileNumber = GetNextFileNumber(FileNumber);
	} else {
	    FileNumber++;
	}
        FileName = GetFileName(FileNumber);
	WriteMail(FileName,&mh,&mb);
#ifdef FACES
	if (strlen(mh.Xface) > 0 ) WriteXFace(&mh);
#endif
	CleanMailMem(&mh,&mb);
	freechar(FileName);
    }
    if (strcmp(mbox,"-") != 0)
	fclose(fd);
}
